@@ -1,6 +1,6 @@
 # -*- Mode: python; coding: utf-8; tab-width: 4; indent-tabs-mode: nil; -*-
 #
-# Copyright (C) 2015 - fossfreedom
+# Copyright (C) 2015 - 2016 - David Mohammed
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -140,6 +140,11 @@ class AltToolbarPlugin(GObject.Object, Peas.Activatable):
             self.gs.PluginKey.SHOW_TOOLTIPS]
         self.enhanced_plugins = self.plugin_settings[
             self.gs.PluginKey.ENHANCED_PLUGINS]
+        self.horiz_categories = self.plugin_settings[
+            self.gs.PluginKey.HORIZ_CATEGORIES]
+        self.app_menu = self.plugin_settings[
+            self.gs.PluginKey.APP_MENU]
+        self.prefer_dark_theme = self.plugin_settings[self.gs.PluginKey.DARK_THEME]
 
         # Add the various application view menus
         self.appshell = ApplicationShell(self.shell)
@@ -416,6 +421,7 @@ class AltToolbarPlugin(GObject.Object, Peas.Activatable):
            sources display-tree signal handler
         """
         print("page changed", page)
+        self.toolbar_type.reset_categories_pos(page)
         self.toolbar_type.reset_toolbar(page)
         self.toolbar_type.reset_entryview(page)
 
