@@ -1,6 +1,6 @@
 # -*- Mode: python; coding: utf-8; tab-width: 4; indent-tabs-mode: nil; -*-
 #
-# Copyright (C) 2015 - fossfreedom
+# Copyright (C) 2015 - 2016 David Mohammed
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -191,9 +191,14 @@ class AltToolbarBase(GObject.Object):
         """
 
         self.startup_completed = True
+        self.reset_categories_pos(self.shell.props.selected_page)
         self.reset_toolbar(self.shell.props.selected_page)
         self.reset_entryview(self.shell.props.selected_page)
 
+        if self.plugin.prefer_dark_theme:
+            settings = Gtk.Settings.get_default()
+            settings.set_property('gtk-application-prefer-dark-theme', True)
+
     def cleanup(self):
         """
           initiate a toolbar cleanup of resources and changes made to rhythmbox
@@ -201,7 +206,8 @@ class AltToolbarBase(GObject.Object):
         """
 
         for page in self._process_entryview:
-            self.disconnect(self._process_entryview[page])
+            self.disconnect(self._process_entryview[page]['size'])
+            self.disconnect(self._process_entryview[page]['changed'])
 
         self.purge_builder_content()
 
@@ -254,6 +260,42 @@ class AltToolbarBase(GObject.Object):
         """
         pass
 
+    def reset_categories_pos(self, page):
+        """
+           whenever a source changes this resets the source categories position
+           reflect the changed source
+           :param page - RBDisplayPage
+        """
+        print("reset categories position")
+        if not page:
+            print("no page")
+            return
+
+        if not hasattr(page.props, 'show_browser'):
+            print("no browser")
+            return
+
+        if not self.plugin.horiz_categories:
+            print("not horizontal")
+            return
+
+        propertyview = self.find(page, 'RBPropertyView', 'by_name')
+
+        if propertyview is None:
+            return
+
+        parent = propertyview.get_parent()
+
+        if isinstance(parent, Gtk.Paned):
+            print("paned")
+            parent.set_orientation(Gtk.Orientation.HORIZONTAL)
+        else:
+            print("not paned")
+            pane = parent.get_parent()
+            print(pane)
+            parent.set_orientation(Gtk.Orientation.VERTICAL)
+            pane.set_orientation(Gtk.Orientation.HORIZONTAL)
+
     def reset_entryview(self, page):
         """
            whenever a source changes this resets the source entryview to
@@ -296,11 +338,12 @@ class AltToolbarBase(GObject.Object):
             if page in self._process_entryview:
                 # disconnect previous signal handler if have been connected
                 # before otherwise we'll trigger stuff when moving columns
-                treeview.disconnect(self._process_entryview[page])
+                treeview.disconnect(self._process_entryview[page]['changed'])
+                treeview.disconnect(self._process_entryview[page]['size'])
 
             # now move columns around depending upon saved values
-            lookup = "pages/page[@name='" + self._safe_string(type(
-                page).__name__) + "']"
+            safe_name = self._safe_string(type(page).__name__)
+            lookup = "pages/page[@name='" + safe_name + "']"
             element = self._entryview_root.find(lookup)
 
             if element is not None:
@@ -331,10 +374,31 @@ class AltToolbarBase(GObject.Object):
                                                                    pos])
                                 break
 
+                                # now reset column widths
+                                # for col in current_cols:
+                                #    safe_col_name = self._safe_string(
+                                # col.props.title)
+
+                                #    lookup = "pages/" + safe_name + "[
+                                # @column='" + \
+                                #             safe_col_name + "']"
+                                #    col_node = self._entryview_root.find(
+                                # lookup)
+
+                                #    if col_node is not None:
+                                #        col.set_fixed_width(int(
+                                # col_node.get("width")))
+
             # now connect new signal handler
-            id = treeview.connect('columns-changed',
-                                  self._entryview_column_changed, page)
-            self._process_entryview[page] = id
+            ids = {}
+            ids['changed'] = treeview.connect('columns-changed',
+                                              self._entryview_column_changed,
+                                              page)
+
+            ids['size'] = treeview.connect('size-allocate',
+                                           self._entryview_size_allocate, page)
+
+            self._process_entryview[page] = ids
 
         # add a short delay otherwise RB will move after us nulling our
         # achievement
@@ -344,6 +408,9 @@ class AltToolbarBase(GObject.Object):
     def _safe_string(self, s):
         return ''.join([i for i in s if i.isalpha()])
 
+    def _entryview_size_allocate(self, treeview, allocation, page):
+        self._entryview_column_changed(treeview, page)
+
     def _entryview_column_changed(self, treeview, page):
         # we basically don't want to process column-changed signals
         # when closing because these are fired by RB during the entry-view
@@ -381,15 +448,16 @@ class AltToolbarBase(GObject.Object):
         def quoted_string(array):
             return ','.join("'{0}'".format(x) for x in array)
 
-        lookup = "pages/page[@name='" + self._safe_string(type(
-            page).__name__) + "']"
+        safe_name = self._safe_string(type(page).__name__)
+        lookup = "pages/page[@name='" + safe_name + "']"
         node = self._entryview_root.find(lookup)
 
+        pages = self._entryview_root.find("pages")
+
         if node is None:
             print("new node")
-            pages = self._entryview_root.find("pages")
             node = SubElement(pages, 'page')
-            node.set("name", self._safe_string(type(page).__name__))
+            node.set("name", safe_name)
 
         arr = []
         cols = treeview.get_columns()
@@ -397,6 +465,17 @@ class AltToolbarBase(GObject.Object):
         for col in cols:
             if col.props.title is not None and col.props.title != "":
                 arr.append(col.props.title)
+                # print (col.get_width())
+                safe_col_name = self._safe_string(col.props.title)
+                lookup = "pages/" + safe_name + "[@column='" + safe_col_name \
+                         + "']"
+                col_node = self._entryview_root.find(lookup)
+
+                if col_node is None:
+                    col_node = SubElement(pages, safe_name)
+                    col_node.set("column", safe_col_name)
+
+                col_node.set("width", str(col.get_width()))
 
         if len(arr) < 2:
             # nothing to do so quit before writing
@@ -771,7 +850,7 @@ class AltToolbarShared(AltToolbarBase):
         """
           register a new controller
         """
-        if not controller in self._controllers:
+        if controller not in self._controllers:
             self._controllers[controller] = controller
 
     def is_controlled(self, source):
@@ -1102,7 +1181,7 @@ class AltToolbarShared(AltToolbarBase):
                 self.__builder_obj_names.append(name)
 
                 if not self.plugin.show_tooltips and obj.get_has_tooltip():
-                    if not name in self._tooltip_exceptions:
+                    if name not in self._tooltip_exceptions:
                         obj.set_has_tooltip(False)
 
     def connect_builder_content(self, builder):
@@ -1187,7 +1266,7 @@ class AltToolbarCompact(AltToolbarShared):
         # appshell = ApplicationShell(self.shell)
         menu = self.shell.props.application.get_menubar()
 
-        if not menu:
+        if not menu or self.plugin.app_menu:
             menu_button = Gtk.MenuButton.new()
             if gtk_version() >= 3.14:
                 symbol = "open-menu-symbolic"
@@ -1277,12 +1356,14 @@ class AltToolbarHeaderBar(AltToolbarShared):
         self._always_visible_sources[source] = source
 
     def _on_key_press(self, widget, event):
+        self.searchbar.handle_event(event)
         keyname = Gdk.keyval_name(event.keyval)
         if keyname == 'Escape' and self.current_search_button:
             self.current_search_button.set_active(False)
         if event.state and Gdk.ModifierType.CONTROL_MASK:
             if keyname == 'f' and self.current_search_button:
-                self.current_search_button.set_active(True)
+                self.current_search_button.set_active(
+                    not self.current_search_button.get_active())
 
     def on_startup(self, *args):
         super(AltToolbarHeaderBar, self).on_startup(*args)
@@ -1339,7 +1420,7 @@ class AltToolbarHeaderBar(AltToolbarShared):
         else:
             GLib.timeout_add(350, delay_hide)
 
-        self.searchbar.set_search_mode(search_button.get_active())
+        self.searchbar.set_search_mode(True)
 
     def set_library_labels(self, song_label=None, category_label=None):
         # locale stuff
@@ -1456,9 +1537,10 @@ class AltToolbarHeaderBar(AltToolbarShared):
         # any source defined controls
         self._end_box_controls.add(self.end_box)
 
-        if (not default.props.gtk_shell_shows_app_menu) or \
-                default.props.gtk_shell_shows_menubar:
-
+        if (not default.props.gtk_shell_shows_app_menu or
+                default.props.gtk_shell_shows_menubar or
+                self.plugin.app_menu) and \
+                        os.environ["XDG_CURRENT_DESKTOP"].find("GNOME") == -1:
             # for environments that dont support app-menus
             menu_button = Gtk.MenuButton.new()
             # menu_button.set_relief(Gtk.ReliefStyle.NONE)
@@ -1475,6 +1557,10 @@ class AltToolbarHeaderBar(AltToolbarShared):
             menu = self.shell.props.application.get_shared_menu('app-menu')
             menu_button.set_menu_model(menu)
             self._end_box_controls.add(menu_button)
+        else:
+            menu = self.shell.props.application.get_shared_menu('app-menu')
+            app = self.shell.props.application
+            app.set_app_menu(menu)
 
         self.headerbar.pack_end(self._end_box_controls)
         self.headerbar.show_all()
@@ -1514,7 +1600,7 @@ class AltToolbarHeaderBar(AltToolbarShared):
         if not ret_generic_bool:
             return
 
-        if not self.shell.props.selected_page in self.sources:
+        if self.shell.props.selected_page not in self.sources:
             ret_bool, controller = \
                 self.is_controlled(self.shell.props.selected_page)
             self.sources[self.shell.props.selected_page] = controller
